@@ -1,4 +1,5 @@
 import asyncio
+import base64
 import concurrent.futures
 import json
 import logging
@@ -10,6 +11,7 @@ from uuid import uuid4
 
 import aioredis
 import boto3
+import snappy
 from aiokafka import AIOKafkaConsumer, AIOKafkaProducer
 from botocore.config import Config
 
@@ -24,6 +26,8 @@ CONTEXT_REDIS_CHANNEL = "redis_channel"
 CONTEXT_REDIS_QUEUE = "redis_queue"
 CONTEXT_SQS_URL = "sqs_queue_url"
 
+SQS_COMPRESSION_SNAPPY = "snappy"
+
 
 class Worker:
 
@@ -176,11 +180,7 @@ class Worker:
                     self._redis = await aioredis.create_redis_pool(
                         self._redis_opts[OPTS_REDIS_URL], timeout=5
                     )
-                    if self._redis_opts.get(OPTS_REDIS_NAME) is not None:
-                        await self._redis.client_setname(
-                            f"{self._redis_opts.get(OPTS_REDIS_NAME)}-{self._uuid}"
-                        )
-                    logger.warning("Redis connected")
+                    asyncio.ensure_future(self.redis_setname_loop())
                 else:
                     logger.warning("Redis opts not supplied")
                 break
@@ -195,19 +195,40 @@ class Worker:
         self._redis_retry = 0
         self._redis_reconnecting = False
 
+    async def redis_setname_loop(self):
+        # during reconnection, the name will not be reset
+        if self._redis_opts.get(OPTS_REDIS_NAME) is not None:
+            name_to_set = f"{self._redis_opts.get(OPTS_REDIS_NAME)}-{self._uuid}"
+            while self._redis is not None and not self.should_exit():
+                try:
+                    existing_name = await self._redis.client_getname()
+                    try:
+                        existing_name = existing_name.decode()  # byte
+                    except Exception:
+                        pass
+                    if existing_name != name_to_set:
+                        await self._redis.client_setname(name_to_set)
+                        logger.warning(
+                            f"redis client name set from: '{existing_name}' to: '{name_to_set}'"
+                        )
+                    await asyncio.sleep(60)
+                except Exception as e:
+                    logger.error(e)
+
     async def destroy_redis(self):
         """
         Don't care if the close fails or timeout
         """
         if self._redis is not None and not self._redis_reconnecting:
             try:
+                logger.warning("Destroying redis")
                 self._redis.close()
-                await asyncio.wait_for(self._redis.wait_closed(), timeout=3)
+                await self._redis.wait_closed()
+                logger.warning("Redis destoryed")
             except Exception as e:
                 logger.error(e)
             finally:
                 self._redis = None
-                logger.warning("Redis destoryed")
 
     def sqs_client(self, queue_url):
         if not queue_url:
@@ -368,10 +389,23 @@ class Worker:
             return
 
     async def send_sqs_message(
-        self, queue_url, message, wait=True, high_throughput=True
+        self,
+        queue_url,
+        message,
+        wait=True,
+        high_throughput=True,
+        compression=SQS_COMPRESSION_SNAPPY,
     ):
         client = self.sqs_client(queue_url)
 
+        message_attributes = None
+        if compression == SQS_COMPRESSION_SNAPPY:
+            # message to keep as a string instead of bytes
+            message = base64.b64encode(snappy.compress(message)).decode()
+            message_attributes = {
+                "compression": {"StringValue": f"{compression}", "DataType": "String"}
+            }
+
         if queue_url.endswith(".fifo"):
             did = f"{time.time()}"
             coro = asyncio.get_running_loop().run_in_executor(
@@ -379,6 +413,7 @@ class Worker:
                 partial(
                     client.send_message,
                     QueueUrl=queue_url,
+                    MessageAttributes=message_attributes,
                     MessageBody=message,
                     MessageDeduplicationId=did,
                     MessageGroupId=did if high_throughput else self._uuid,
@@ -387,7 +422,12 @@ class Worker:
         else:
             coro = asyncio.get_running_loop().run_in_executor(
                 None,
-                partial(client.send_message, QueueUrl=queue_url, MessageBody=message,),
+                partial(
+                    client.send_message,
+                    QueueUrl=queue_url,
+                    MessageAttributes=message_attributes,
+                    MessageBody=message,
+                ),
             )
 
         if wait:
@@ -399,16 +439,26 @@ class Worker:
         queue_url = self._sqs_opts[OPTS_SQS_QUEUE_URL]
         client = self.sqs_client(queue_url)
 
-        return await asyncio.get_running_loop().run_in_executor(
+        response = await asyncio.get_running_loop().run_in_executor(
             None,
             partial(
                 client.receive_message,
                 QueueUrl=queue_url,
+                MessageAttributeNames=["compression"],
                 MaxNumberOfMessages=num_message,
                 WaitTimeSeconds=15,
             ),
         )
 
+        for message in response.get("Messages", []):
+            compression = message.get("MessageAttributes", {}).get("compression")
+            if compression and compression.get("StringValue") == SQS_COMPRESSION_SNAPPY:
+                message["Body"] = snappy.uncompress(
+                    base64.b64decode(message["Body"])
+                ).decode()
+
+        return response
+
     async def delete_sqs_message(self, receipt_handle):
         queue_url = self._sqs_opts[OPTS_SQS_QUEUE_URL]
         client = self.sqs_client(queue_url)
@@ -420,9 +470,9 @@ class Worker:
             ),
         )
 
-    async def receive_redis_queue(self):
+    async def receive_redis_queue(self, timeout=0):
         queue_key = self._redis_opts[OPTS_REDIS_QUEUE_KEY]
-        return await self._redis.blpop(queue_key)
+        return await self._redis.blpop(queue_key, timeout=timeout)
 
     async def send_result(
         self, result, context=None, result_channel=None, wait_for_sqs=True
